/*
 * XdccDownloader - Source.java - Copyright © 2013 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.irc;

import com.google.common.base.Objects;
import com.google.common.base.Optional;

/**
 * Container for reply source information.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class Source {

	/** The nickname. */
	private final Optional<String> nick;

	/** The username. */
	private final Optional<String> username;

	/** The server. */
	private final String server;

	/**
	 * Creates a new source.
	 *
	 * @param nick
	 * 		The nickname
	 * @param username
	 * 		The username
	 * @param server
	 * 		The server
	 */
	Source(Optional<String> nick, Optional<String> username, String server) {
		this.nick = nick;
		this.username = username;
		this.server = server;
	}

	//
	// ACCESSORS
	//

	/**
	 * Returns the nickname of the source.
	 *
	 * @return The nickname of the source
	 */
	public Optional<String> nick() {
		return nick;
	}

	/**
	 * Returns the username of the source.
	 *
	 * @return The username of the source
	 */
	public Optional<String> username() {
		return username;
	}

	/**
	 * Returns the server of the source.
	 *
	 * @return The server of the source
	 */
	public String server() {
		return server;
	}

	//
	// OBJECT METHODS
	//

	@Override
	public boolean equals(Object object) {
		if (!(object instanceof Source)) {
			return false;
		}
		Source source = (Source) object;
		if (!Objects.equal(nick(), source.nick())) {
			return false;
		}
		if (!Objects.equal(username(), source.username())) {
			return false;
		}
		if (!Objects.equal(server(), source.server())) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		return (nick.isPresent() ? nick().get() + "!" : "") + (username().isPresent() ? username.get() + "@" : "") + server;
	}

	//
	// STATIC METHODS
	//

	/**
	 * Parses a source from the given string.
	 *
	 * @param source
	 * 		The string to which to parse the source from
	 * @return The parsed source
	 */
	public static Source parseSource(String source) {
		int exclamationMark = source.indexOf('!');
		int atSign = source.indexOf('@', exclamationMark);

		if ((exclamationMark == -1) || (atSign == -1)) {
			return new Source(Optional.<String>absent(), Optional.<String>absent(), source);
		}
		return new Source(Optional.of(source.substring(0, exclamationMark)), Optional.of(source.substring(exclamationMark + 1, atSign)), source.substring(atSign + 1));
	}

}
