/*
 * XdccDownloader - DccSendReceived.java - Copyright © 2013 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.irc.event;

import net.pterodactylus.irc.Connection;
import net.pterodactylus.irc.Source;

/**
 * Notifies a listener that a DCC ACCEPT message was received. A DCC ACCEPT is a
 * response to a {@link Connection#sendDccResume(String, String, int, long) DCC
 * RESUME} request.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class DccAcceptReceived extends AbstractConnectionEvent {

	/** The source of the DCC SEND. */
	private final Source source;

	/** The name of the file being offered. */
	private final String filename;

	/** The port number of the source. */
	private final int port;

	/** The position at which the download will start. */
	private final long position;

	/**
	 * Creates a new DCC ACCEPT received event.
	 *
	 * @param connection
	 * 		The connetion the event occured on
	 * @param source
	 * 		The source offering the file
	 * @param filename
	 * 		The name of the file being offered
	 * @param port
	 * 		The port number of the source
	 * @param position
	 * 		The size of the file being offered ({@code -1} for unknown size)
	 */
	public DccAcceptReceived(Connection connection, Source source, String filename, int port, long position) {
		super(connection);
		this.source = source;
		this.filename = filename;
		this.port = port;
		this.position = position;
	}

	//
	// ACCESSORS
	//

	/**
	 * Returns the source offering the file.
	 *
	 * @return The source offering the file
	 */
	public Source source() {
		return source;
	}

	/**
	 * Returns the name of the file being offered.
	 *
	 * @return The name of the file being offered
	 */
	public String filename() {
		return filename;
	}

	/**
	 * Returns the position at which the download will start.
	 *
	 * @return The position at which the download will start
	 */
	public long position() {
		return position;
	}

	/**
	 * Returns the port number of the source.
	 *
	 * @return The port number of the source
	 */
	public int port() {
		return port;
	}

}
