/*
 * XdccDownloader - Main.java - Copyright © 2013 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.xdcc.main;

import static com.lexicalscope.jewel.cli.CliFactory.parseArguments;

import java.io.File;
import java.util.Collection;
import java.util.HashSet;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.concurrent.Executors;

import net.pterodactylus.irc.ConnectionFactory;
import net.pterodactylus.irc.DefaultConnectionFactory;
import net.pterodactylus.xdcc.core.Core;
import net.pterodactylus.xdcc.data.Channel;
import net.pterodactylus.xdcc.data.Download;
import net.pterodactylus.xdcc.data.Network;
import net.pterodactylus.xdcc.data.Network.NetworkBuilder;
import net.pterodactylus.xdcc.data.Network.ServerBuilder;
import net.pterodactylus.xdcc.ui.stdin.CommandReader;
import net.pterodactylus.xdcc.ui.stdin.NetworkAdapter;

import com.google.common.eventbus.AsyncEventBus;
import com.google.common.eventbus.EventBus;
import com.lexicalscope.jewel.cli.Option;
import org.codehaus.jackson.map.ObjectMapper;

/**
 * Xudocci main class.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public class Main {

	private static final String VERSION = "0.1-SNAPSHOT";

	public static String version() {
		return VERSION;
	}

	public static void main(String... arguments) throws Exception {

		Parameters parameters = parseArguments(Parameters.class, arguments);

		EventBus eventBus = new AsyncEventBus(Executors.newSingleThreadExecutor());
		Configuration configuration = new ObjectMapper().readValue(new File(parameters.getConfiguration()), Configuration.class);
		ConnectionFactory connectionFactory = new DefaultConnectionFactory(eventBus);

		Core core = new Core(eventBus, connectionFactory, configuration.getTemporaryDirectory(), configuration.getFinalDirectory());
		eventBus.register(core);

		for (Configuration.Network network : configuration.getNetworks()) {
			NetworkBuilder networkBuilder = Network.builder(network.getName());
			for (Configuration.Network.Server server : network.getServers()) {
				ServerBuilder serverBuilder = networkBuilder.addServer();
				serverBuilder.at(server.getHostname());
				for (int unencryptedPort : server.getUnencryptedPorts()) {
					serverBuilder.port(unencryptedPort);
				}
				for (int encryptedPort : server.getEncryptedPorts()) {
					serverBuilder.sslPort(encryptedPort);
				}
				serverBuilder.endServer();
			}
			Network createdNetwork = networkBuilder.build();
			for (String channel : network.getChannels()) {
				core.addChannel(new Channel(createdNetwork, channel));
			}
		}

		Collection<Download> failedDownloads = new CopyOnWriteArraySet<>();

		NetworkAdapter networkAcceptor = new NetworkAdapter(eventBus, (reader, writer) -> new CommandReader(core, reader, writer, failedDownloads), configuration.getTelnetPort());
		networkAcceptor.start();

		core.start();
	}

	/**
	 * Defines the command-line parameters.
	 *
	 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
	 */
	private interface Parameters {

		@Option(longName = "configuration", description = "The configuration file", defaultValue = "configuration.json")
		String getConfiguration();

	}

}
